@@ -19,7 +19,9 @@ import java.io.IOException;
 import java.text.MessageFormat;
 import java.text.ParseException;
 
+import java.util.Collection;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -29,6 +31,7 @@ import java.util.Vector;
 import org.freedesktop.DBus;
 import org.freedesktop.dbus.exceptions.DBusException;
 import org.freedesktop.dbus.exceptions.DBusExecutionException;
+import org.freedesktop.dbus.exceptions.NotConnected;
 
 import cx.ath.matthew.debug.Debug;
 
@@ -43,6 +46,112 @@ import cx.ath.matthew.debug.Debug;
  */
 public class DBusConnection extends AbstractConnection
 {
+	/**
+	 * Add addresses of peers to a set which will watch for them to
+	 * disappear and automatically remove them from the set.
+	 */
+	public class PeerSet implements Set<String>, DBusSigHandler<DBus.NameOwnerChanged>
+	{
+		private Set<String> addresses;
+		public PeerSet()
+		{
+			addresses = new TreeSet<String>();
+			try { 
+				addSigHandler(new DBusMatchRule(DBus.NameOwnerChanged.class, null, null), this);
+			} catch (DBusException DBe) {
+				if (EXCEPTION_DEBUG && Debug.debug) Debug.print(Debug.ERR, DBe);
+			}
+		}
+		public void handle(DBus.NameOwnerChanged noc)
+		{
+			if (Debug.debug)
+				Debug.print(Debug.DEBUG, "Received NameOwnerChanged("+noc.name+","+noc.old_owner+","+noc.new_owner+")");
+			if ("".equals(noc.new_owner) && addresses.contains(noc.name)) 
+				remove(noc.name);
+		}
+		public boolean add(String address)
+		{
+			if (Debug.debug)
+				Debug.print(Debug.DEBUG, "Adding "+address);
+			synchronized (addresses) {
+				return addresses.add(address);
+			}
+		}
+		public boolean addAll(Collection<? extends String> addresses)
+		{
+			synchronized (this.addresses) {
+				return this.addresses.addAll(addresses);
+			}
+		}
+		public void clear()
+		{
+			synchronized (addresses) {
+				addresses.clear();
+			}
+		}
+		public boolean contains(Object o)
+		{
+			return addresses.contains(o);
+		}
+		public boolean containsAll(Collection<?> os)
+		{
+			return addresses.containsAll(os);
+		}
+		public boolean equals(Object o)
+		{
+			if (o instanceof PeerSet)
+				return ((PeerSet) o).addresses.equals(addresses);
+			else return false;
+		}
+		public int hashCode()
+		{
+			return addresses.hashCode();
+		}
+		public boolean isEmpty()
+		{
+			return addresses.isEmpty();
+		}
+		public Iterator<String> iterator()
+		{
+			return addresses.iterator();
+		}
+		public boolean remove(Object o)
+		{
+			if (Debug.debug)
+				Debug.print(Debug.DEBUG, "Removing "+o);
+			synchronized(addresses) {
+				return addresses.remove(o);
+			}
+		}
+		public boolean removeAll(Collection<?> os)
+		{
+			synchronized(addresses) {
+				return addresses.removeAll(os);
+			}
+		}
+		public boolean retainAll(Collection<?> os)
+		{
+			synchronized(addresses) {
+				return addresses.retainAll(os);
+			}
+		}
+		public int size()
+		{
+			return addresses.size();
+		}
+		public Object[] toArray()
+		{
+			synchronized(addresses) {
+				return addresses.toArray();
+			}
+		}
+		public <T> T[] toArray(T[] a)
+		{
+			synchronized(addresses) {
+				return addresses.toArray(a);
+			}
+		}
+	}
    private class _sighandler implements DBusSigHandler<DBusSignal>
    {
       public void handle(DBusSignal s)
@@ -157,6 +266,7 @@ public class DBusConnection extends AbstractConnection
    
       try {
          transport = new Transport(addr, AbstractConnection.TIMEOUT);
+			connected = true;
       } catch (IOException IOe) {
          if (EXCEPTION_DEBUG && Debug.debug) Debug.print(Debug.ERR, IOe);            
          disconnect();
@@ -188,9 +298,11 @@ public class DBusConnection extends AbstractConnection
    @SuppressWarnings("unchecked")
    DBusInterface dynamicProxy(String source, String path) throws DBusException
    {
+		if (Debug.debug) Debug.print(Debug.INFO, "Introspecting "+path+" on "+source+" for dynamic proxy creation");
       try {
          DBus.Introspectable intro = getRemoteObject(source, path, DBus.Introspectable.class);
          String data = intro.Introspect();
+			if (Debug.debug) Debug.print(Debug.VERBOSE, "Got introspection data: "+data);
          String[] tags = data.split("[<>]");
          Vector<String> ifaces = new Vector<String>();
          for (String tag: tags) {
@@ -200,6 +312,7 @@ public class DBusConnection extends AbstractConnection
          }
          Vector<Class<? extends Object>> ifcs = new Vector<Class<? extends Object>>();
          for(String iface: ifaces) {
+				if (Debug.debug) Debug.print(Debug.DEBUG, "Trying interface "+iface);
             int j = 0;
             while (j >= 0) {
                try {
@@ -247,6 +360,27 @@ public class DBusConnection extends AbstractConnection
       return dynamicProxy(source, path);
    }
 
+   /** 
+    * Release a bus name.
+	 * Releases the name so that other people can use it
+    * @param busname The name to release. MUST be in dot-notation like "org.freedesktop.local"
+    * @throws DBusException If the busname is incorrectly formatted.
+    */
+   public void releaseBusName(String busname) throws DBusException
+   {
+      if (!busname.matches(BUSNAME_REGEX)||busname.length() > MAX_NAME_LENGTH)
+         throw new DBusException(_("Invalid bus name"));
+      synchronized (this.busnames) {
+         UInt32 rv;
+         try { 
+            rv = _dbus.ReleaseName(busname);
+         } catch (DBusExecutionException DBEe) {
+            if (EXCEPTION_DEBUG && Debug.debug) Debug.print(Debug.ERR, DBEe);
+            throw new DBusException(DBEe.getMessage());
+         }
+         this.busnames.remove(busname);
+      }
+   }
    /** 
     * Request a bus name.
     * Request the well known name that this should respond to on the Bus.
@@ -278,6 +412,7 @@ public class DBusConnection extends AbstractConnection
          this.busnames.add(busname);
       }
    }
+
    /**
     * Returns the unique name of this connection.
     */
@@ -507,9 +642,11 @@ public class DBusConnection extends AbstractConnection
                handledSignals.remove(key);
                try {
                   _dbus.RemoveMatch(rule.toString());
+               } catch (NotConnected NC) {
+                  if (EXCEPTION_DEBUG && Debug.debug) Debug.print(Debug.ERR, NC);
                } catch (DBusExecutionException DBEe) {
                   if (EXCEPTION_DEBUG && Debug.debug) Debug.print(Debug.ERR, DBEe);
-                  throw new DBusException(DBEe.getMessage());
+						throw new DBusException(DBEe.getMessage());
                }
             }
          } 
